from io import BytesIO
from PIL import Image, ImageOps
from io import BytesIO


class ThumbnailCreator:
    def create_thumbnail_name(self, image_uri: str, height: int) -> str:
        filename_without_extension = image_uri.split(".")[0]
        return f"{filename_without_extension}_thumbnail_{height}.jpg"

    def create_thumbnail(self, image_data, height: int) -> bytes:
        image = Image.open(BytesIO(image_data))
        image = ImageOps.exif_transpose(image)
        if image is None:
            raise ValueError("Invalid image data.")
        aspect_ratio = image.width / image.height
        new_width = int(height * aspect_ratio)
        image.thumbnail((new_width, height), resample=Image.Resampling.BILINEAR)
        thumbnail_data = BytesIO()
        image.save(thumbnail_data, format="JPEG")
        thumbnail_data.seek(0)
        thumbnail_data = thumbnail_data.read()
        return thumbnail_data



from abc import abstractmethod, ABC
from ..model import RenderedFile, RendererParameters


class BaseRenderer(ABC):
    @abstractmethod
    def render(self, parameters: RendererParameters) -> list[RenderedFile] | RenderedFile:
        raise NotImplementedError



from .azure_blob_storage import AzureBlobSourceDataProvider, AzureBlobStorageProvider
from .local_storage import LocalSourceDataProvider, LocalStorageProvider
from .base_provider import BaseSourceDataProvider, BaseStorageProvider

from ..model import LocalStorageConfig, AzureBlobStorageConfig


def get_source_provider(data_source_config: LocalStorageConfig | AzureBlobStorageConfig) -> BaseSourceDataProvider:
    if data_source_config.type == "azure-blob":
        return AzureBlobSourceDataProvider(data_source_config)  # type: ignore
    elif data_source_config.type == "local":
        return LocalSourceDataProvider(data_source_config)  # type: ignore
    else:
        raise ValueError(f"Unsupported data source type: {data_source_config.type}")


def get_storage_provider(storage_config: LocalStorageConfig | AzureBlobStorageConfig) -> BaseStorageProvider:
    if storage_config.type == "azure-blob":
        return AzureBlobStorageProvider(storage_config)  # type: ignore
    elif storage_config.type == "local":
        return LocalStorageProvider(storage_config)  # type: ignore
    else:
        raise ValueError(f"Unsupported storage type: {storage_config.type}")
